# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/log'

module Sqreen
  class CBTree
    include Enumerable
    # Callbacks tree:
    # class
    # methods
    # position

    def initialize
      @by_class = {}
    end

    def add(cb)
      @by_class[cb.klass] = {} unless @by_class[cb.klass]

      cb_klass = @by_class[cb.klass]
      cb_klass[cb.method] ||= [nil, nil, nil]

      methods = cb_klass[cb.method]

      if cb.pre?
        methods[0] ||= []
        add_to_array(methods[0], cb)
      end
      if cb.post?
        methods[1] ||= []
        add_to_array(methods[1], cb)
      end
      if cb.failing? # rubocop:disable Style/GuardClause
        methods[2] ||= []
        add_to_array(methods[2], cb)
      end
    end

    def remove(cb)
      by_meth = @by_class[cb.klass]
      return unless by_meth
      types = by_meth[cb.method]

      if cb.pre? && types[0]
        types[0].delete(cb)
        types[0] = nil if types[0].empty?
      end
      if cb.post? && types[1]
        types[1].delete(cb)
        types[1] = nil if types[1].empty?
      end
      if cb.failing? && types[2] # rubocop:disable Style/GuardClause
        types[2].delete(cb)
        types[2] = nil if types[2].empty?
      end
    end

    def get(klass, method)
      k = @by_class[klass]
      unless k
        Sqreen.log.debug { format('Error: no cb registered for class %s (%s)', klass.inspect, klass.class) }
        Sqreen.log.debug { inspect }
        return [nil, nil, nil]
      end
      cbs = k[method]
      unless cbs
        Sqreen.log.debug { format('Error: no cbs registered for method %s.%s', klass, method) }
        Sqreen.log.debug { inspect }
        return [nil, nil, nil]
      end
      cbs
    rescue StandardError => e
      Sqreen.log.warn "we catched an exception getting cbs: #{e.inspect}"
      Sqreen::RemoteException.record(e)
      [nil, nil, nil]
    end

    def each
      @by_class.each_value do |values|
        values.each_value do |cbs|
          cbs.each do |cb_ary|
            next unless cb_ary
            cb_ary.each do |cb|
              yield cb
            end
          end
        end
      end
    end

    private

    def add_to_array(arr, cb)
      # find last element with prio equal or smaller; insert after
      pos = arr.size
      arr.reverse_each do |arr_cb|
        break if arr_cb.priority <= cb.priority
        pos -= 1
      end

      arr.insert(pos, cb)
    end
  end
end
