# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/shrink_wrap'
require 'sqreen/middleware'
require 'sqreen/error_handling_middleware'
require 'sqreen/rails_middleware'

module Sqreen
  module Dependency
    module Rails
      module_function

      def required?
        Sqreen::Dependency.const_exist?('Rails::Application')
      end

      def server?
        Sqreen::Dependency.const_exist?('Rails::Server') && ObjectSpace.each_object(::Rails::Server).count > 0
      end

      def inspect_middlewares
        return unless ::Rails.application.middleware.respond_to?(:map)

        Sqreen.log.debug { "Middlewares: " << ::Rails.application.middleware.map(&:inspect).inspect }
      end

      def insert_sqreen_middlewares
        Sqreen.log.debug { 'Inserting Sqreen middlewares for Rails' }
        app = ::Rails.application
        app.middleware.insert(0, Sqreen::ShrinkWrap)
        app.middleware.insert_after(::Rack::Runtime, Sqreen::Middleware)
        app.middleware.insert_after(::ActionDispatch::DebugExceptions, Sqreen::RailsMiddleware)
        app.middleware.insert_after(::ActionDispatch::DebugExceptions, Sqreen::ErrorHandlingMiddleware)
      end
    end
  end
end
