# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'logger'

module Sqreen
  module Ecosystem
    # Configured by the ecosystem client
    module DispatchTable
      class << self
        # data consumption
        # argument: +Sqreen::Kit::Signals::Signal+
        # see +Sqreen::EcosystemIntegration::SignalConsumption#consume_signal+
        attr_accessor :consume_signal

        # argument: block taking a Rack::Request
        # see +Sqreen::EcosystemIntegration::RequestLifecycleTracking#add_start_observer+
        attr_accessor :add_request_start_listener

        attr_accessor :fetch_logger

        # argument: callback taking:
        # * the method to instrument
        # * A Hash{Symbol=>Proc} with the advice. The proc takes the
        # arguments and the ball, so these details of the instrumentation
        # implementation leak through the abstraction
        # see +Sqreen::EcosystemIntegration::InstrumentationService+
        attr_accessor :instrument

        def reset
          instance_variables.each do |ia|
            instance_variable_set(ia, nil)
          end

          # set default logger
          logger = ::Logger.new(STDERR)
          logger.level = ::Logger::WARN
          logger.progname = 'sqreen-ecosystem'
          self.fetch_logger = proc { logger }
        end
      end

      reset
    end
  end
end
