# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/ecosystem/dispatch_table'
require 'sqreen/ecosystem/loggable'
require 'sqreen/kit/configuration'
require 'sqreen/kit/signals/specialized/sqreen_exception'

module Sqreen
  module Ecosystem
    module ExceptionReporting
      include Loggable

      private

      # @param [String] message
      # @param [Exception] e
      def report_exception(message, e)
        logger.warn { "#{message}: #{e.message}" }
        logger.debug { e.backtrace.map { |x| "  #{x}" }.join("\n") }

        signal = Sqreen::Kit::Signals::Specialized::SqreenException.new(
          ruby_exception: e,
          source: Sqreen::Kit::Configuration.default_source
        )

        DispatchTable.consume_signal[signal]
      end
    end
  end
end
