# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/ecosystem/transaction_storage'
require 'sqreen/ecosystem/loggable'

module Sqreen
  module Ecosystem
    module ModuleApi
      module TransactionStorage
        class TxLocalVariables
          class << self
            class << self
              include Sqreen::Ecosystem::Loggable

              private

              def attr_reader(attr, _opts = {})
                define_method attr do
                  tx_storage = Ecosystem::TransactionStorage.fetch_thread_local
                  return unless tx_storage
                  tx_storage[attr]
                end
              end

              def attr_accessor(attr, opts = {})
                # reader
                attr_reader attr, opts

                # writer (2 variants)
                do_assign = proc do |value|
                  tx_storage = Ecosystem::TransactionStorage.fetch_thread_local
                  unless tx_storage
                    logger.debug do
                      "Assignment of tx local attribute #{attr} to #{value} has no effect"
                    end
                    return
                  end

                  tx_storage[attr] = value
                end

                if opts.fetch(:allow_overwrite, false)
                  define "#{attr}=", &do_assign
                else
                  define_method "#{attr}=" do |value|
                    cur = public_send(attr)
                    unless cur.nil?
                      raise "Cannot override value of #{attr} from #{cur} with #{value}"
                    end

                    do_assign.call(value)
                  end
                end
              end
            end # TxLocalVariables.singleton_class.singleton_class

            # usage:
            # attr_reader :xxx

            # in the future, we'll possibly need to expose the full
            # TransactionStorage to the modules, at least if we don't
            # opt for a more structured fashion of data exchange between
            # the modules.
          end # TxLocalVariables.singleton_class
        end # TxLocalVariables

        def tx_local_vars
          TxLocalVariables
        end
      end # TransactionStorage module
    end
  end
end
