# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/ecosystem/module_registry'
require 'sqreen/ecosystem/transaction_storage'
require 'sqreen/ecosystem/module_api/signal_producer'

module Sqreen
  module Ecosystem
    class TracingIdSetup
      # @param [Array<Sqreen::Ecosystem::ModuleApi::SignalProducer>] signal_producer_modules
      def initialize(signal_producer_modules)
        @modules = signal_producer_modules
        @tracing_id_prefix = nil
      end

      def setup_modules
        inject_out_of_tx_tracing_id_gen
      end

      attr_writer :tracing_id_prefix

      private

      def inject_out_of_tx_tracing_id_gen
        @modules.each do |mod|
          mod.tracing_id_producer = method(:generate_tracing_id)
        end
      end

      def generate_tracing_id
        return nil unless @tracing_id_prefix
        "#{@tracing_id_prefix}.#{SecureRandom.uuid}"
      end
    end
  end
end
