# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/log'

module Sqreen
  # Ruby default formatter modified to display current thread_id
  class FormatterWithTid
    # TODO: constant name
    Format = "%s, [%s#%d.%s] %5s -- %s: %s\n".freeze
    DatetimeFormat = '%Y-%m-%dT%H:%M:%S.%6N '.freeze

    attr_accessor :datetime_format

    def initialize
      @datetime_format = nil
    end

    def call(severity, time, progname, msg)
      format(
        Format,
        severity[0..0], format_datetime(time), $$,
        Thread.current.object_id.to_s(36),
        severity, progname, msg2str(msg),
      )
    end

    private

    def format_datetime(time)
      time.strftime(DatetimeFormat)
    end

    def msg2str(msg)
      case msg
      when ::String
        msg
      when ::Exception
        "#{msg.message} (#{msg.class})\n" << (msg.backtrace || []).join("\n")
      else
        msg.inspect
      end
    end
  end
end
