# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/log'
require 'sqreen/exception'
require 'singleton'

module Sqreen
  module Js
    class JsService
      include ::Singleton

      def initialize
        detect_adapter
      end

      # @return [Sqreen::Js::ExecutableJs]
      def prepare(rule_name, code)
        raise 'Not online' unless online?
        @adapter.preprocess(rule_name, code)
      end

      def online?
        @online
      end

      def variant
        @adapter.variant_name
      end

      private

      def detect_adapter
        @online = try_sq_mini_racer || try_rhino

        Sqreen.log.info "JS engine online: #{variant}" if @online
      end

      def try_sq_mini_racer
        gem = Gem.loaded_specs['sq_mini_racer']
        unless gem
          Sqreen.log.info "sq_mini_racer gem not detected"
          return false
        end

        require 'sqreen/mini_racer'
        require 'sqreen/js/mini_racer_adapter'
        @adapter = MiniRacerAdapter.new(true)
      rescue LoadError => e
        Sqreen.log.warn "Failed loading sq_mini_racer: #{e}"
        false
      end

      def try_rhino
        gem = Gem.loaded_specs['therubyrhino']
        unless gem
          Sqreen.log.info "therubyrhino gem not detected"
          return false
        end

        require 'rhino'
        require 'sqreen/js/execjs_adapter'
        @adapter = ExecjsAdapter.new
      rescue LoadError => e
        Sqreen.log.warn "Failed loading rhino: #{e}"
        false
      end
    end
  end
end
