# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/kit/signals/point'
require 'sqreen/kit/signals/dto_helper'

# reference: https://github.com/sqreen/SignalsSchemas/blob/master/schemas/payload/attack/2020-01-01T00_00_00_000Z/schema.cue

module Sqreen
  module Kit
    module Signals
      module Specialized
      end
    end
  end
end

class Sqreen::Kit::Signals::Specialized::Attack < Sqreen::Kit::Signals::Point
  add_mandatory_attrs :source, :time, :payload

  validate_str_attr :signal_name, /\Asq\.agent\.attack\..+\z/
  validate_str_attr :source, /\Asqreen:rule:[a-f0-9]{40}:.+\z/

  def initialize(values = {})
    self.payload_schema = Payload::SCHEMA_VERSION
    self.time = values[:time] || Time.now
    super
  end

  def payload=(payload)
    unless payload.is_a?(Payload)
      raise ArgumentError, "Payload should be a #{Payload}"
    end
    super
  end

  class Payload
    include Sqreen::Kit::Signals::DtoHelper

    SCHEMA_VERSION = 'attack/2020-01-01T00:00:00.000Z'.freeze

    add_mandatory_attrs :test, :block, :infos

    # all are mandatory

    # @return [Boolean]
    attr_accessor :test

    # @return [Boolean]
    attr_accessor :block

    # @return [Hash{String|Symbol=>Object}]
    attr_accessor :infos
  end
end
