# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/rules/rule_cb'

module Sqreen
  module Rules
    # Save request context for handling further down
    class RecordRequestContext < RuleCB
      WHITELISTED_METRIC = 'whitelisted'.freeze
      def initialize(*args)
        super(*args)
        @overtimeable = false
      end

      def whitelisted?
        false
      end

      def pre(_inst, args, _budget = nil, &_block)
        framework.store_request(args[0])
        wh = framework.whitelisted_match
        if wh
          unless Sqreen.features.key?('whitelisted_metric') &&
                 !Sqreen.features['whitelisted_metric']
            record_observation(WHITELISTED_METRIC, wh, 1)
          end
          Sqreen.log.debug { "Request was whitelisted because of #{wh}" }
        end
        advise_action(nil)
      end

      def post(rv, _inst, args, _budget = nil, &_block)
        framework.store_response(rv, args[0])
        framework.clean_request
        advise_action(nil)
      end

      def failing(_exception, _inst, _args, _budget = nil, &_block)
        framework.clean_request
        advise_action(nil)
      end
    end
  end
end
